@@ -1,4 +1,4 @@
-// Copyright (c) 2016-2022 The Regents of the University of Michigan
+// Copyright (c) 2016-2023 The Regents of the University of Michigan
 // Part of GSD, released under the BSD 2-Clause License.
 
 #include <sys/stat.h>
@@ -58,16 +58,22 @@ enum
     GSD_INITIAL_FRAME_INDEX_SIZE = 16
     };
 
-/// Size of write buffer
+/// Initial size of write buffer
 enum
     {
-    GSD_WRITE_BUFFER_SIZE = 16 * 1024 * 1024
+    GSD_INITIAL_WRITE_BUFFER_SIZE = 1024
     };
 
-/// Size of copy buffer
+/// Default maximum size of write buffer
 enum
     {
-    GSD_COPY_BUFFER_SIZE = 128 * 1024
+    GSD_DEFAULT_MAXIMUM_WRITE_BUFFER_SIZE = 64 * 1024 * 1024
+    };
+
+/// Default number of index entries to buffer
+enum
+    {
+    GSD_DEFAULT_INDEX_ENTRIES_TO_BUFFER = 256 * 1024
     };
 
 /// Size of hash map
@@ -959,7 +965,13 @@ inline static int gsd_expand_file_index(struct gsd_handle* handle, size_t size_r
         }
 
     // allocate the copy buffer
-    char* buf = malloc(GSD_COPY_BUFFER_SIZE);
+    uint64_t copy_buffer_size
+        = GSD_DEFAULT_INDEX_ENTRIES_TO_BUFFER * sizeof(struct gsd_index_entry);
+    if (copy_buffer_size > size_old * sizeof(struct gsd_index_entry))
+        {
+        copy_buffer_size = size_old * sizeof(struct gsd_index_entry);
+        }
+    char* buf = malloc(copy_buffer_size);
 
     // write the current index to the end of the file
     int64_t new_index_location = lseek(handle->fd, 0, SEEK_END);
@@ -968,8 +980,8 @@ inline static int gsd_expand_file_index(struct gsd_handle* handle, size_t size_r
     size_t old_index_bytes = size_old * sizeof(struct gsd_index_entry);
     while (total_bytes_written < old_index_bytes)
         {
-        size_t bytes_to_copy = GSD_COPY_BUFFER_SIZE;
-        if (old_index_bytes - total_bytes_written < GSD_COPY_BUFFER_SIZE)
+        size_t bytes_to_copy = copy_buffer_size;
+        if (old_index_bytes - total_bytes_written < copy_buffer_size)
             {
             bytes_to_copy = old_index_bytes - total_bytes_written;
             }
@@ -1000,13 +1012,13 @@ inline static int gsd_expand_file_index(struct gsd_handle* handle, size_t size_r
         }
 
     // fill the new index space with 0s
-    gsd_util_zero_memory(buf, GSD_COPY_BUFFER_SIZE);
+    gsd_util_zero_memory(buf, copy_buffer_size);
 
     size_t new_index_bytes = size_new * sizeof(struct gsd_index_entry);
     while (total_bytes_written < new_index_bytes)
         {
-        size_t bytes_to_copy = GSD_COPY_BUFFER_SIZE;
-        if (new_index_bytes - total_bytes_written < GSD_COPY_BUFFER_SIZE)
+        size_t bytes_to_copy = copy_buffer_size;
+        if (new_index_bytes - total_bytes_written < copy_buffer_size)
             {
             bytes_to_copy = new_index_bytes - total_bytes_written;
             }
@@ -1114,7 +1126,7 @@ inline static int gsd_flush_write_buffer(struct gsd_handle* handle)
     // reset write_buffer for new data
     handle->write_buffer.size = 0;
 
-    // move buffer_index entries to file_index
+    // Move buffer_index entries to frame_index.
     size_t i;
     for (i = 0; i < handle->buffer_index.size; i++)
         {
@@ -1570,7 +1582,7 @@ inline static int gsd_initialize_handle(struct gsd_handle* handle)
             return retval;
             }
 
-        retval = gsd_byte_buffer_allocate(&handle->write_buffer, GSD_WRITE_BUFFER_SIZE);
+        retval = gsd_byte_buffer_allocate(&handle->write_buffer, GSD_INITIAL_WRITE_BUFFER_SIZE);
         if (retval != GSD_SUCCESS)
             {
             return retval;
@@ -1584,6 +1596,10 @@ inline static int gsd_initialize_handle(struct gsd_handle* handle)
             }
         }
 
+    handle->pending_index_entries = 0;
+    handle->maximum_write_buffer_size = GSD_DEFAULT_MAXIMUM_WRITE_BUFFER_SIZE;
+    handle->index_entries_to_buffer = GSD_DEFAULT_INDEX_ENTRIES_TO_BUFFER;
+
     return GSD_SUCCESS;
     }
 
@@ -1794,10 +1810,21 @@ int gsd_close(struct gsd_handle* handle)
         return GSD_ERROR_INVALID_ARGUMENT;
         }
 
+    int retval;
+
+    if (handle->open_flags != GSD_OPEN_READONLY)
+        {
+        retval = gsd_flush(handle);
+        if (retval != GSD_SUCCESS)
+            {
+            return retval;
+            }
+        }
+
     // save the fd so we can use it after freeing the handle
     int fd = handle->fd;
 
-    int retval = gsd_index_buffer_free(&handle->file_index);
+    retval = gsd_index_buffer_free(&handle->file_index);
     if (retval != GSD_SUCCESS)
         {
         return retval;
@@ -1877,8 +1904,27 @@ int gsd_end_frame(struct gsd_handle* handle)
         return GSD_ERROR_FILE_MUST_BE_WRITABLE;
         }
 
-    // increment the frame counter
     handle->cur_frame++;
+    handle->pending_index_entries = 0;
+
+    if (handle->frame_index.size > 0 || handle->buffer_index.size > handle->index_entries_to_buffer)
+        {
+        return gsd_flush(handle);
+        }
+
+    return GSD_SUCCESS;
+    }
+
+int gsd_flush(struct gsd_handle* handle)
+    {
+    if (handle == NULL)
+        {
+        return GSD_ERROR_INVALID_ARGUMENT;
+        }
+    if (handle->open_flags == GSD_OPEN_READONLY)
+        {
+        return GSD_ERROR_FILE_MUST_BE_WRITABLE;
+        }
 
     // flush the namelist buffer
     int retval = gsd_flush_name_buffer(handle);
@@ -1894,13 +1940,27 @@ int gsd_end_frame(struct gsd_handle* handle)
         return retval;
         }
 
-    // write the frame index to the file
-    if (handle->frame_index.size > 0)
+    // sync the data before writing the index
+    retval = fsync(handle->fd);
+    if (retval != 0)
+        {
+        return GSD_ERROR_IO;
+        }
+
+    // Write the frame index to the file, excluding the index entries that are part of the current
+    // frame.
+    if (handle->pending_index_entries > handle->frame_index.size)
+        {
+        return GSD_ERROR_INVALID_ARGUMENT;
+        }
+    uint64_t index_entries_to_write = handle->frame_index.size - handle->pending_index_entries;
+
+    if (index_entries_to_write > 0)
         {
         // ensure there is enough space in the index
-        if ((handle->file_index.size + handle->frame_index.size) > handle->file_index.reserved)
+        if ((handle->file_index.size + index_entries_to_write) > handle->file_index.reserved)
             {
-            gsd_expand_file_index(handle, handle->file_index.size + handle->frame_index.size);
+            gsd_expand_file_index(handle, handle->file_index.size + index_entries_to_write);
             }
 
         // sort the index before writing
@@ -1914,7 +1974,7 @@ int gsd_end_frame(struct gsd_handle* handle)
         int64_t write_pos = handle->header.index_location
                             + sizeof(struct gsd_index_entry) * handle->file_index.size;
 
-        size_t bytes_to_write = sizeof(struct gsd_index_entry) * handle->frame_index.size;
+        size_t bytes_to_write = sizeof(struct gsd_index_entry) * index_entries_to_write;
         ssize_t bytes_written
             = gsd_io_pwrite_retry(handle->fd, handle->frame_index.data, bytes_to_write, write_pos);
 
@@ -1927,14 +1987,24 @@ int gsd_end_frame(struct gsd_handle* handle)
         // add the entries to the file index
         memcpy(handle->file_index.data + handle->file_index.size,
                handle->frame_index.data,
-               sizeof(struct gsd_index_entry) * handle->frame_index.size);
+               sizeof(struct gsd_index_entry) * index_entries_to_write);
 #endif
 
         // update size of file index
-        handle->file_index.size += handle->frame_index.size;
+        handle->file_index.size += index_entries_to_write;
 
-        // clear the frame index
-        handle->frame_index.size = 0;
+        // Clear the frame index, keeping those in the current unfinished frame.
+        if (handle->pending_index_entries > 0)
+            {
+            for (uint64_t i = 0; i < handle->pending_index_entries; i++)
+                {
+                handle->frame_index.data[i]
+                    = handle->frame_index
+                          .data[handle->frame_index.size - handle->pending_index_entries];
+                }
+            }
+
+        handle->frame_index.size = handle->pending_index_entries;
         }
 
     return GSD_SUCCESS;
@@ -1994,10 +2064,10 @@ int gsd_write_chunk(struct gsd_handle* handle,
     size_t size = N * M * gsd_sizeof_type(type);
 
     // decide whether to write this chunk to the buffer or straight to disk
-    if (size < handle->write_buffer.reserved / 2)
+    if (size < handle->maximum_write_buffer_size)
         {
         // flush the buffer if this entry won't fit
-        if (size > (handle->write_buffer.reserved - handle->write_buffer.size))
+        if (size > (handle->maximum_write_buffer_size - handle->write_buffer.size))
             {
             gsd_flush_write_buffer(handle);
             }
@@ -2050,6 +2120,7 @@ int gsd_write_chunk(struct gsd_handle* handle,
         handle->file_size += bytes_written;
         }
 
+    handle->pending_index_entries++;
     return GSD_SUCCESS;
     }
 
@@ -2077,6 +2148,14 @@ gsd_find_chunk(struct gsd_handle* handle, uint64_t frame, const char* name)
         {
         return NULL;
         }
+    if (handle->open_flags != GSD_OPEN_READONLY)
+        {
+        int retval = gsd_flush(handle);
+        if (retval != GSD_SUCCESS)
+            {
+            return NULL;
+            }
+        }
 
     // find the id for the given name
     uint16_t match_id = gsd_name_id_map_find(&handle->name_map, name);
@@ -2167,6 +2246,14 @@ int gsd_read_chunk(struct gsd_handle* handle, void* data, const struct gsd_index
         {
         return GSD_ERROR_INVALID_ARGUMENT;
         }
+    if (handle->open_flags != GSD_OPEN_READONLY)
+        {
+        int retval = gsd_flush(handle);
+        if (retval != GSD_SUCCESS)
+            {
+            return retval;
+            }
+        }
 
     size_t size = chunk->N * chunk->M * gsd_sizeof_type((enum gsd_type)chunk->type);
     if (size == 0)
@@ -2258,6 +2345,14 @@ gsd_find_matching_chunk_name(struct gsd_handle* handle, const char* match, const
         {
         return NULL;
         }
+    if (handle->open_flags != GSD_OPEN_READONLY)
+        {
+        int retval = gsd_flush(handle);
+        if (retval != GSD_SUCCESS)
+            {
+            return NULL;
+            }
+        }
 
     // return nothing found if the name buffer is corrupt
     if (handle->file_names.data.data[handle->file_names.data.reserved - 1] != 0)
@@ -2475,6 +2570,48 @@ int gsd_upgrade(struct gsd_handle* handle)
     return GSD_SUCCESS;
     }
 
+uint64_t gsd_get_maximum_write_buffer_size(struct gsd_handle* handle)
+    {
+    if (handle == NULL)
+        {
+        return 0;
+        }
+    return handle->maximum_write_buffer_size;
+    }
+
+int gsd_set_maximum_write_buffer_size(struct gsd_handle* handle, uint64_t size)
+    {
+    if (handle == NULL || size == 0)
+        {
+        return GSD_ERROR_INVALID_ARGUMENT;
+        }
+
+    handle->maximum_write_buffer_size = size;
+
+    return GSD_SUCCESS;
+    }
+
+uint64_t gsd_get_index_entries_to_buffer(struct gsd_handle* handle)
+    {
+    if (handle == NULL)
+        {
+        return 0;
+        }
+    return handle->index_entries_to_buffer;
+    }
+
+int gsd_set_index_entries_to_buffer(struct gsd_handle* handle, uint64_t number)
+    {
+    if (handle == NULL || number == 0)
+        {
+        return GSD_ERROR_INVALID_ARGUMENT;
+        }
+
+    handle->index_entries_to_buffer = number;
+
+    return GSD_SUCCESS;
+    }
+
 // undefine windows wrapper macros
 #ifdef _WIN32
 #undef lseek
