@@ -1,17 +1,20 @@
-# Copyright (c) 2016-2022 The Regents of the University of Michigan
+# Copyright (c) 2016-2023 The Regents of the University of Michigan
 # Part of GSD, released under the BSD 2-Clause License.
 
 """Read and write HOOMD schema GSD files.
 
 :py:mod:`gsd.hoomd` reads and writes GSD files with the ``hoomd`` schema.
 
-* `open` - Open a hoomd schema GSD file.
 * `HOOMDTrajectory` - Read and write hoomd schema GSD files.
-* `Snapshot` - Store the state of a single frame.
+* `Frame` - Store the state of a single frame.
 
-    * `ConfigurationData` - Store configuration data in a snapshot.
-    * `ParticleData` - Store particle data in a snapshot.
-    * `BondData` - Store topology data in a snapshot.
+  * `ConfigurationData` - Store configuration data in a frame.
+  * `ParticleData` - Store particle data in a frame.
+  * `BondData` - Store topology data in a frame.
+
+* `open` - Open a hoomd schema GSD file.
+* `read_log` - Read log from a hoomd schema GSD file into a dict of time-series
+  arrays.
 
 See Also:
     See :ref:`hoomd-examples` for full examples.
@@ -39,7 +42,7 @@ logger = logging.getLogger('gsd.hoomd')
 class ConfigurationData(object):
     """Store configuration data.
 
-    Use the `Snapshot.configuration` attribute of a to access the configuration.
+    Use the `Frame.configuration` attribute of a to access the configuration.
 
     Attributes:
         step (int): Time step of this frame (:chunk:`configuration/step`).
@@ -105,18 +108,18 @@ class ConfigurationData(object):
 class ParticleData(object):
     """Store particle data chunks.
 
-    Use the `Snapshot.particles` attribute of a to access the particles.
+    Use the `Frame.particles` attribute of a to access the particles.
 
     Instances resulting from file read operations will always store array
     quantities in `numpy.ndarray` objects of the defined types. User created
-    snapshots may provide input data that can be converted to a `numpy.ndarray`.
+    frames may provide input data that can be converted to a `numpy.ndarray`.
 
     See Also:
         `hoomd.State` for a full description of how HOOMD interprets this
         data.
 
     Attributes:
-        N (int): Number of particles in the snapshot (:chunk:`particles/N`).
+        N (int): Number of particles in the frame (:chunk:`particles/N`).
 
         types (tuple[str]):
             Names of the particle types (:chunk:`particles/types`).
@@ -256,13 +259,13 @@ class ParticleData(object):
 class BondData(object):
     """Store bond data chunks.
 
-    Use the `Snapshot.bonds`, `Snapshot.angles`, `Snapshot.dihedrals`,
-    `Snapshot.impropers`, and `Snapshot.pairs` attributes to access the bond
+    Use the `Frame.bonds`, `Frame.angles`, `Frame.dihedrals`,
+    `Frame.impropers`, and `Frame.pairs` attributes to access the bond
     topology.
 
     Instances resulting from file read operations will always store array
     quantities in `numpy.ndarray` objects of the defined types. User created
-    snapshots may provide input data that can be converted to a `numpy.ndarray`.
+    frames may provide input data that can be converted to a `numpy.ndarray`.
 
     See Also:
         `hoomd.State` for a full description of how HOOMD interprets this
@@ -285,7 +288,7 @@ class BondData(object):
 
     Attributes:
         N (int): Number of bonds/angles/dihedrals/impropers/pairs in the
-          snapshot
+          frame
           (:chunk:`bonds/N`, :chunk:`angles/N`, :chunk:`dihedrals/N`,
           :chunk:`impropers/N`, :chunk:`pairs/N`).
 
@@ -348,18 +351,18 @@ class BondData(object):
 class ConstraintData(object):
     """Store constraint data.
 
-    Use the `Snapshot.constraints` attribute to access the constraints.
+    Use the `Frame.constraints` attribute to access the constraints.
 
     Instances resulting from file read operations will always store array
     quantities in `numpy.ndarray` objects of the defined types. User created
-    snapshots may provide input data that can be converted to a `numpy.ndarray`.
+    frames may provide input data that can be converted to a `numpy.ndarray`.
 
     See Also:
         `hoomd.State` for a full description of how HOOMD interprets this
         data.
 
     Attributes:
-        N (int): Number of constraints in the snapshot (:chunk:`constraints/N`).
+        N (int): Number of constraints in the frame (:chunk:`constraints/N`).
 
         value ((*N*, ) `numpy.ndarray` of ``numpy.float32``):
             Constraint length (:chunk:`constraints/value`).
@@ -404,8 +407,8 @@ class ConstraintData(object):
             self.group = self.group.reshape([self.N, self.M])
 
 
-class Snapshot(object):
-    """Snapshot of a system state.
+class Frame:
+    """System state at one point in time.
 
     Attributes:
         configuration (`ConfigurationData`): Configuration data.
@@ -465,8 +468,8 @@ class Snapshot(object):
         ]
 
     def validate(self):
-        """Validate all contained snapshot data."""
-        logger.debug('Validating Snapshot')
+        """Validate all contained frame data."""
+        logger.debug('Validating Frame')
 
         self.configuration.validate()
         self.particles.validate()
@@ -713,23 +716,22 @@ class HOOMDTrajectory(object):
         """The number of frames in the trajectory."""
         return self.file.nframes
 
-    def append(self, snapshot):
-        """Append a snapshot to a hoomd gsd file.
+    def append(self, frame):
+        """Append a frame to a hoomd gsd file.
 
         Args:
-            snapshot (:py:class:`Snapshot`): Snapshot to append.
+            frame (:py:class:`Frame`): Frame to append.
 
-        Write the given snapshot to the file at the current frame and increase
+        Write the given frame to the file at the current frame and increase
         the frame counter. Do not write any fields that are ``None``. For all
         non-``None`` fields, scan them and see if they match the initial frame
         or the default value. If the given data differs, write it out to the
         frame. If it is the same, do not write it out as it can be instantiated
         either from the value at the initial frame or the default value.
         """
-        logger.debug('Appending snapshot to hoomd trajectory: '
-                     + str(self.file))
+        logger.debug('Appending frame to hoomd trajectory: ' + str(self.file))
 
-        snapshot.validate()
+        frame.validate()
 
         # want the initial frame specified as a reference to detect if chunks
         # need to be written
@@ -746,9 +748,9 @@ class HOOMDTrajectory(object):
                 'constraints',
                 'pairs',
         ]:
-            container = getattr(snapshot, path)
+            container = getattr(frame, path)
             for name in container._default_value:
-                if self._should_write(path, name, snapshot):
+                if self._should_write(path, name, frame):
                     logger.debug('writing data chunk: ' + path + '/' + name)
                     data = getattr(container, name)
 
@@ -770,11 +772,11 @@ class HOOMDTrajectory(object):
                     self.file.write_chunk(path + '/' + name, data)
 
         # write state data
-        for state, data in snapshot.state.items():
+        for state, data in frame.state.items():
             self.file.write_chunk('state/' + state, data)
 
         # write log data
-        for log, data in snapshot.log.items():
+        for log, data in frame.log.items():
             self.file.write_chunk('log/' + log, data)
 
         self.file.end_frame()
@@ -789,19 +791,19 @@ class HOOMDTrajectory(object):
         self.file.close()
         del self._initial_frame
 
-    def _should_write(self, path, name, snapshot):
+    def _should_write(self, path, name, frame):
         """Test if we should write a given data chunk.
 
         Args:
             path (str): Path part of the data chunk.
             name (str): Name part of the data chunk.
-            snapshot (:py:class:`Snapshot`): Snapshot data is from.
+            frame (:py:class:`Frame`): Frame data is from.
 
         Returns:
             False if the data matches that in the initial frame. False
             if the data matches all default values. True otherwise.
         """
-        container = getattr(snapshot, path)
+        container = getattr(frame, path)
         data = getattr(container, name)
 
         if data is None:
@@ -815,7 +817,14 @@ class HOOMDTrajectory(object):
                              + '/' + name)
                 return False
 
-        if numpy.array_equiv(data, container._default_value[name]) \
+        matches_default_value = False
+        if name == 'types':
+            matches_default_value = data == container._default_value[name]
+        else:
+            matches_default_value = numpy.array_equiv(
+                data, container._default_value[name])
+
+        if matches_default_value \
                 and not self.file.chunk_exists(frame=0, name=path + '/' + name):
             logger.debug('skipping data chunk, default value: ' + path + '/'
                          + name)
@@ -827,34 +836,27 @@ class HOOMDTrajectory(object):
         """Append each item of the iterable to the file.
 
         Args:
-            iterable: An iterable object the provides :py:class:`Snapshot`
+            iterable: An iterable object the provides :py:class:`Frame`
                 instances. This could be another HOOMDTrajectory, a generator
-                that modifies snapshots, or a list of snapshots.
+                that modifies frames, or a list of frames.
         """
         for item in iterable:
             self.append(item)
 
-    def read_frame(self, idx):
+    def _read_frame(self, idx):
         """Read the frame at the given index from the file.
 
         Args:
             idx (int): Frame index to read.
 
         Returns:
-            `Snapshot` with the frame data
+            `Frame` with the frame data
 
         Replace any data chunks not present in the given frame with either data
         from frame 0, or initialize from default values if not in frame 0. Cache
         frame 0 data to avoid file read overhead. Return any default data as
         non-writable numpy arrays.
-
-        .. deprecated:: v2.5
         """
-        warnings.warn("Deprecated, trajectory[idx]", DeprecationWarning)
-        return self._read_frame(idx)
-
-    def _read_frame(self, idx):
-        """Implements read_frame."""
         if idx >= len(self):
             raise IndexError
 
@@ -863,40 +865,41 @@ class HOOMDTrajectory(object):
         if self._initial_frame is None and idx != 0:
             self._read_frame(0)
 
-        snap = Snapshot()
+        frame = Frame()
         # read configuration first
         if self.file.chunk_exists(frame=idx, name='configuration/step'):
             step_arr = self.file.read_chunk(frame=idx,
                                             name='configuration/step')
-            snap.configuration.step = step_arr[0]
+            frame.configuration.step = step_arr[0]
         else:
             if self._initial_frame is not None:
-                snap.configuration.step = self._initial_frame.configuration.step
+                frame.configuration.step = \
+                    self._initial_frame.configuration.step
             else:
-                snap.configuration.step = \
-                    snap.configuration._default_value['step']
+                frame.configuration.step = \
+                    frame.configuration._default_value['step']
 
         if self.file.chunk_exists(frame=idx, name='configuration/dimensions'):
             dimensions_arr = self.file.read_chunk(
                 frame=idx, name='configuration/dimensions')
-            snap.configuration.dimensions = dimensions_arr[0]
+            frame.configuration.dimensions = dimensions_arr[0]
         else:
             if self._initial_frame is not None:
-                snap.configuration.dimensions = \
+                frame.configuration.dimensions = \
                     self._initial_frame.configuration.dimensions
             else:
-                snap.configuration.dimensions = \
-                    snap.configuration._default_value['dimensions']
+                frame.configuration.dimensions = \
+                    frame.configuration._default_value['dimensions']
 
         if self.file.chunk_exists(frame=idx, name='configuration/box'):
-            snap.configuration.box = self.file.read_chunk(
+            frame.configuration.box = self.file.read_chunk(
                 frame=idx, name='configuration/box')
         else:
             if self._initial_frame is not None:
-                snap.configuration.box = self._initial_frame.configuration.box
+                frame.configuration.box = self._initial_frame.configuration.box
             else:
-                snap.configuration.box = \
-                    snap.configuration._default_value['box']
+                frame.configuration.box = \
+                    frame.configuration._default_value['box']
 
         # then read all groups that have N, types, etc...
         for path in [
@@ -908,7 +911,7 @@ class HOOMDTrajectory(object):
                 'constraints',
                 'pairs',
         ]:
-            container = getattr(snap, path)
+            container = getattr(frame, path)
             if self._initial_frame is not None:
                 initial_frame_container = getattr(self._initial_frame, path)
 
@@ -979,25 +982,25 @@ class HOOMDTrajectory(object):
                     container.__dict__[name].flags.writeable = False
 
         # read state data
-        for state in snap._valid_state:
+        for state in frame._valid_state:
             if self.file.chunk_exists(frame=idx, name='state/' + state):
-                snap.state[state] = self.file.read_chunk(frame=idx,
-                                                         name='state/' + state)
+                frame.state[state] = self.file.read_chunk(frame=idx,
+                                                          name='state/' + state)
 
         # read log data
         logged_data_names = self.file.find_matching_chunk_names('log/')
         for log in logged_data_names:
             if self.file.chunk_exists(frame=idx, name=log):
-                snap.log[log[4:]] = self.file.read_chunk(frame=idx, name=log)
+                frame.log[log[4:]] = self.file.read_chunk(frame=idx, name=log)
             else:
                 if self._initial_frame is not None:
-                    snap.log[log[4:]] = self._initial_frame.log[log[4:]]
+                    frame.log[log[4:]] = self._initial_frame.log[log[4:]]
 
         # store initial frame
         if self._initial_frame is None and idx == 0:
-            self._initial_frame = snap
+            self._initial_frame = frame
 
-        return snap
+        return frame
 
     def __getitem__(self, key):
         """Index trajectory frames.
@@ -1034,7 +1037,7 @@ class HOOMDTrajectory(object):
         self.file.close()
 
 
-def open(name, mode='rb'):
+def open(name, mode='r'):
     """Open a hoomd schema GSD file.
 
     The return value of `open` can be used as a context manager.
@@ -1047,38 +1050,29 @@ def open(name, mode='rb'):
         `HOOMDTrajectory` instance that accesses the file **name** with the
         given **mode**.
 
-    Valid values for mode:
+    Valid values for ``mode``:
 
     +------------------+---------------------------------------------+
     | mode             | description                                 |
     +==================+=============================================+
-    | ``'rb'``         | Open an existing file for reading.          |
+    | ``'r'``          | Open an existing file for reading.          |
     +------------------+---------------------------------------------+
-    | ``'rb+'``        | Open an existing file for reading and       |
+    | ``'r+'``         | Open an existing file for reading and       |
     |                  | writing.                                    |
     +------------------+---------------------------------------------+
-    | ``'wb'``         | Open a file for reading and writing.        |
+    | ``'w'``          | Open a file for reading and writing.        |
     |                  | Creates the file if needed, or overwrites   |
     |                  | an existing file.                           |
     +------------------+---------------------------------------------+
-    | ``'wb+'``        | Open a file for reading and writing.        |
-    |                  | Creates the file if needed, or overwrites   |
-    |                  | an existing file.                           |
-    +------------------+---------------------------------------------+
-    | ``'xb'``         | Create a gsd file exclusively and opens it  |
+    | ``'x'``          | Create a gsd file exclusively and opens it  |
     |                  | for reading and writing.                    |
     |                  | Raise :py:exc:`FileExistsError`             |
     |                  | if it already exists.                       |
     +------------------+---------------------------------------------+
-    | ``'xb+'``        | Create a gsd file exclusively and opens it  |
-    |                  | for reading and writing.                    |
-    |                  | Raise :py:exc:`FileExistsError`             |
-    |                  | if it already exists.                       |
-    +------------------+---------------------------------------------+
-    | ``'ab'``         | Open an existing file for reading and       |
-    |                  | writing. Does *not* create or overwrite     |
-    |                  | existing files.                             |
+    | ``'a'``          | Open a file for reading and writing.        |
+    |                  | Creates the file if it doesn't exist.       |
     +------------------+---------------------------------------------+
+
     """
     if fl is None:
         raise RuntimeError("file layer module is not available")
@@ -1087,8 +1081,91 @@ def open(name, mode='rb'):
 
     gsdfileobj = fl.open(name=str(name),
                          mode=mode,
-                         application='gsd.hoomd ' + gsd.__version__,
+                         application='gsd.hoomd ' + gsd.version.version,
                          schema='hoomd',
                          schema_version=[1, 4])
 
     return HOOMDTrajectory(gsdfileobj)
+
+
+def read_log(name, scalar_only=False):
+    """Read log from a hoomd schema GSD file into a dict of time-series arrays.
+
+    Args:
+        name (str): File name to open.
+        scalar_only (bool): Set to `True` to include only scalar log values.
+
+    The log data includes :chunk:`configuration/step` and all matching
+    :chunk:`log/user_defined`, :chunk:`log/bonds/user_defined`, and
+    :chunk:`log/particles/user_defined` quantities in the file.
+
+    Returns:
+        `dict`
+
+    Note:
+        `read_log` issues a `RuntimeWarning` when there are no matching
+        ``log/`` quantities in the file.
+
+    Caution:
+        `read_log` requires that a logged quantity has the same shape in all
+        frames. Use `open` and `Frame.log` to read files where the shape
+        changes from frame to frame.
+
+    To create a *pandas* ``DataFrame`` with the logged data:
+
+    .. ipython:: python
+
+        import pandas
+
+        df = pandas.DataFrame(gsd.hoomd.read_log('log-example.gsd',
+                                                  scalar_only=True))
+        df
+    """
+    if fl is None:
+        raise RuntimeError("file layer module is not available")
+    if gsd is None:
+        raise RuntimeError("gsd module is not available")
+
+    with fl.open(name=str(name),
+                 mode='r',
+                 application='gsd.hoomd ' + gsd.version.version,
+                 schema='hoomd',
+                 schema_version=[1, 4]) as gsdfileobj:
+
+        logged_data_names = gsdfileobj.find_matching_chunk_names('log/')
+        # Always log timestep associated with each log entry
+        logged_data_names.insert(0, 'configuration/step')
+        if len(logged_data_names) == 1:
+            warnings.warn('No logged data in file: ' + str(name),
+                          RuntimeWarning)
+
+        logged_data_dict = dict()
+        for log in logged_data_names:
+            log_exists_frame_0 = gsdfileobj.chunk_exists(frame=0, name=log)
+            is_configuration_step = log == 'configuration/step'
+
+            if log_exists_frame_0 or is_configuration_step:
+                if is_configuration_step and not log_exists_frame_0:
+                    # handle default configuration step on frame 0
+                    tmp = numpy.array([0], dtype=numpy.uint64)
+                else:
+                    tmp = gsdfileobj.read_chunk(frame=0, name=log)
+
+                if scalar_only and not tmp.shape[0] == 1:
+                    continue
+                if tmp.shape[0] == 1:
+                    logged_data_dict[log] = numpy.full(
+                        fill_value=tmp[0], shape=(gsdfileobj.nframes,))
+                else:
+                    logged_data_dict[log] = numpy.tile(
+                        tmp,
+                        (gsdfileobj.nframes,) + tuple(1 for _ in tmp.shape))
+
+            for idx in range(1, gsdfileobj.nframes):
+                for log in logged_data_dict.keys():
+                    if not gsdfileobj.chunk_exists(frame=idx, name=log):
+                        continue
+                    logged_data_dict[log][idx] = gsdfileobj.read_chunk(
+                        frame=idx, name=log)
+
+    return logged_data_dict
