@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2022 The Regents of the University of Michigan
+# Copyright (c) 2016-2023 The Regents of the University of Michigan
 # Part of GSD, released under the BSD 2-Clause License.
 
 """Pytest fixtures common to all tests."""
@@ -7,7 +7,7 @@ import pytest
 import collections
 
 Mode = collections.namedtuple('Mode', 'read write')
-mode_list = [Mode('rb', 'wb'), Mode('rb+', 'wb+')]
+mode_list = [Mode('r', 'w'), Mode('a', 'x'), Mode('r', 'a')]
 
 
 def open_mode_name(mode):
