@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2022 The Regents of the University of Michigan
+# Copyright (c) 2016-2023 The Regents of the University of Michigan
 # Part of GSD, released under the BSD 2-Clause License.
 
 """Test gsd.fl."""
@@ -17,9 +17,9 @@ import sys
 test_path = pathlib.Path(os.path.realpath(__file__)).parent
 
 
-def test_create(tmp_path):
+def test_create(tmp_path, open_mode):
     """Test creation of GSD files."""
-    gsd.fl.open(mode='xb',
+    gsd.fl.open(mode=open_mode.write,
                 name=tmp_path / "test_create.gsd",
                 application="test_create",
                 schema="none",
@@ -40,18 +40,18 @@ def test_create(tmp_path):
 ])
 def test_dtype(tmp_path, typ):
     """Test all supported data types."""
-    data1d = numpy.array([1, 2, 3, 4, 5, 10012], dtype=typ)
+    data1d = numpy.array([1, 2, 3, 4, 5, 127], dtype=typ)
     data2d = numpy.array([[10, 20], [30, 40], [50, 80]], dtype=typ)
     data_zero = numpy.array([], dtype=typ)
 
-    gsd.fl.open(mode='xb',
+    gsd.fl.open(mode='x',
                 name=tmp_path / "test_dtype.gsd",
                 application="test_dtype",
                 schema="none",
                 schema_version=[1, 2])
 
     with gsd.fl.open(name=tmp_path / "test_dtype.gsd",
-                     mode='wb',
+                     mode='w',
                      application="test_dtype",
                      schema="none",
                      schema_version=[1, 2]) as f:
@@ -61,7 +61,7 @@ def test_dtype(tmp_path, typ):
         f.end_frame()
 
     with gsd.fl.open(name=tmp_path / "test_dtype.gsd",
-                     mode='rb',
+                     mode='r',
                      application="test_dtype",
                      schema="none",
                      schema_version=[1, 2]) as f:
@@ -119,7 +119,7 @@ def test_metadata(tmp_path, open_mode):
     with gsd.pygsd.GSDFile(
             file=open(str(tmp_path / 'test_metadata.gsd'), mode='rb')) as f:
         assert f.name == str(tmp_path / 'test_metadata.gsd')
-        assert f.mode == 'rb'
+        assert f.mode == 'r'
         assert f.application == 'test_metadata'
         assert f.schema == 'none'
         assert f.schema_version == (1, 2)
@@ -140,11 +140,11 @@ def test_append(tmp_path, open_mode):
     nframes = 1024
 
     with gsd.fl.open(name=tmp_path / 'test_append.gsd',
-                     mode='ab',
+                     mode='a',
                      application='test_append',
                      schema='none',
                      schema_version=[1, 2]) as f:
-        assert f.mode == 'ab'
+        assert f.mode == 'a'
         for i in range(nframes):
             data[0] = i
             f.write_chunk(name='data1', data=data)
@@ -166,8 +166,7 @@ def test_append(tmp_path, open_mode):
 
     # test again with pygsd
     with gsd.pygsd.GSDFile(
-            file=open(str(tmp_path
-                          / 'test_append.gsd'), mode=open_mode.read)) as f:
+            file=open(str(tmp_path / 'test_append.gsd'), mode='rb')) as f:
         assert f.nframes == nframes
         for i in range(nframes):
             data1 = f.read_chunk(frame=i, name='data1')
@@ -268,7 +267,7 @@ def test_readonly_errors(tmp_path, open_mode):
 
     data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.int64)
     with gsd.fl.open(name=tmp_path / 'test_readonly_errors.gsd',
-                     mode='rb',
+                     mode='r',
                      application='test_readonly_errors',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -299,8 +298,7 @@ def test_fileio_errors(tmp_path, open_mode):
                         schema='none',
                         schema_version=[1, 2])
 
-        with open(str(tmp_path / 'test_fileio_errors.gsd'),
-                  open_mode.write) as f:
+        with open(str(tmp_path / 'test_fileio_errors.gsd'), 'wb') as f:
             f.write(b'test')
 
         with pytest.raises(RuntimeError):
@@ -316,7 +314,7 @@ def test_dtype_errors(tmp_path, open_mode):
     with pytest.raises(Exception):
         data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.bool_)
 
-        with gsd.fl.open(name=tmp_path / 'test_dtype_errors.gsd',
+        with gsd.fl.open(name=tmp_path / 'test_dtype_errors1.gsd',
                          mode=open_mode.write,
                          application='test_dtype_errors',
                          schema='none',
@@ -327,7 +325,7 @@ def test_dtype_errors(tmp_path, open_mode):
     with pytest.raises(Exception):
         data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.float16)
 
-        with gsd.fl.open(name=tmp_path / 'test_dtype_errors.gsd',
+        with gsd.fl.open(name=tmp_path / 'test_dtype_errors2.gsd',
                          mode=open_mode.write,
                          application='test_dtype_errors',
                          schema='none',
@@ -338,7 +336,7 @@ def test_dtype_errors(tmp_path, open_mode):
     with pytest.raises(Exception):
         data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.complex64)
 
-        with gsd.fl.open(name=tmp_path / 'test_dtype_errors.gsd',
+        with gsd.fl.open(name=tmp_path / 'test_dtype_errors3.gsd',
                          mode=open_mode.write,
                          application='test_dtype_errors',
                          schema='none',
@@ -349,7 +347,7 @@ def test_dtype_errors(tmp_path, open_mode):
     with pytest.raises(Exception):
         data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.complex128)
 
-        with gsd.fl.open(name=tmp_path / 'test_dtype_errors.gsd',
+        with gsd.fl.open(name=tmp_path / 'test_dtype_errors4.gsd',
                          mode=open_mode.write,
                          application='test_dtype_errors',
                          schema='none',
@@ -363,11 +361,11 @@ def test_truncate(tmp_path):
     data = numpy.ascontiguousarray(numpy.random.random(size=(1000, 3)),
                                    dtype=numpy.float32)
     with gsd.fl.open(name=tmp_path / 'test_truncate.gsd',
-                     mode='wb',
+                     mode='w',
                      application='test_truncate',
                      schema='none',
                      schema_version=[1, 2]) as f:
-        assert f.mode == 'wb'
+        assert f.mode == 'w'
         for i in range(10):
             f.write_chunk(name='data', data=data)
             f.end_frame()
@@ -384,12 +382,12 @@ def test_truncate(tmp_path):
         f.end_frame()
 
     with gsd.fl.open(name=tmp_path / 'test_truncate.gsd',
-                     mode='rb',
+                     mode='r',
                      application='test_truncate',
                      schema='none',
                      schema_version=[1, 2]) as f:
         assert f.name == str(tmp_path / 'test_truncate.gsd')
-        assert f.mode == 'rb'
+        assert f.mode == 'r'
         assert f.application == 'test_truncate'
         assert f.schema == 'none'
         assert f.schema_version == (1, 2)
@@ -434,7 +432,7 @@ def test_open(tmp_path):
     data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.int64)
 
     with gsd.fl.open(name=tmp_path / 'test.gsd',
-                     mode='xb',
+                     mode='x',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -442,7 +440,7 @@ def test_open(tmp_path):
         f.end_frame()
 
     with gsd.fl.open(name=tmp_path / 'test_2.gsd',
-                     mode='xb+',
+                     mode='x',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -451,7 +449,7 @@ def test_open(tmp_path):
         f.read_chunk(0, name='chunk1')
 
     with gsd.fl.open(name=tmp_path / 'test.gsd',
-                     mode='wb',
+                     mode='w',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -459,7 +457,7 @@ def test_open(tmp_path):
         f.end_frame()
 
     with gsd.fl.open(name=tmp_path / 'test.gsd',
-                     mode='wb+',
+                     mode='w',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -468,7 +466,7 @@ def test_open(tmp_path):
         f.read_chunk(0, name='chunk1')
 
     with gsd.fl.open(name=tmp_path / 'test.gsd',
-                     mode='ab',
+                     mode='a',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -476,7 +474,7 @@ def test_open(tmp_path):
         f.end_frame()
 
     with gsd.fl.open(name=tmp_path / 'test.gsd',
-                     mode='rb',
+                     mode='r',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -484,7 +482,7 @@ def test_open(tmp_path):
         f.read_chunk(1, name='chunk1')
 
     with gsd.fl.open(name=tmp_path / 'test.gsd',
-                     mode='rb+',
+                     mode='r+',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -635,7 +633,7 @@ def test_gsd_v1_read():
 
     # test with the C implemantation
     with gsd.fl.open(name=test_path / 'test_gsd_v1.gsd',
-                     mode='rb',
+                     mode='r',
                      application='test_gsd_v1',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -675,7 +673,7 @@ def test_gsd_v1_upgrade_read(tmp_path, open_mode):
     shutil.copy(test_path / 'test_gsd_v1.gsd', tmp_path / 'test_gsd_v1.gsd')
 
     with gsd.fl.open(name=tmp_path / 'test_gsd_v1.gsd',
-                     mode='rb+',
+                     mode='r+',
                      application='test_gsd_v1',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -750,7 +748,7 @@ def test_gsd_v1_write(tmp_path, open_mode):
 
     # test that we can write new entries to the file
     with gsd.fl.open(name=tmp_path / 'test_gsd_v1.gsd',
-                     mode='rb+',
+                     mode='r+',
                      application='test_gsd_v1',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -819,7 +817,7 @@ def test_gsd_v1_upgrade_write(tmp_path, open_mode):
 
     # test that we can write new entries to the file
     with gsd.fl.open(name=tmp_path / 'test_gsd_v1.gsd',
-                     mode='rb+',
+                     mode='r+',
                      application='test_gsd_v1',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -884,9 +882,8 @@ def test_zero_size(tmp_path, open_mode):
         assert data_read.dtype == numpy.float32
 
     # test again with pygsd
-    with gsd.pygsd.GSDFile(
-            file=open(str(tmp_path
-                          / 'test_zero.gsd'), mode=open_mode.read)) as f:
+    with gsd.pygsd.GSDFile(file=open(str(tmp_path
+                                         / 'test_zero.gsd'), mode='rb')) as f:
         assert f.nframes == 1
         data_read = f.read_chunk(frame=0, name='data')
         assert data_read.shape == (0,)
@@ -902,7 +899,7 @@ def test_utf8(tmp_path):
     fname = '中文.gsd'
 
     with gsd.fl.open(name=tmp_path / fname,
-                     mode='xb',
+                     mode='x',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -914,7 +911,7 @@ def test_utf8(tmp_path):
     assert fname in dir_list
 
     with gsd.fl.open(name=tmp_path / fname,
-                     mode='wb',
+                     mode='w',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
@@ -922,19 +919,19 @@ def test_utf8(tmp_path):
         f.end_frame()
 
     with gsd.fl.open(name=tmp_path / fname,
-                     mode='rb',
+                     mode='r',
                      application='test_open',
                      schema='none',
                      schema_version=[1, 2]) as f:
         f.read_chunk(0, name='chunk1')
 
 
-@pytest.mark.parametrize('mode', ['wb', 'wb+', 'ab', 'rb+', 'xb', 'xb+'])
+@pytest.mark.parametrize('mode', ['w', 'x', 'a', 'r+'])
 def test_read_write(tmp_path, mode):
     """Test that data chunks can read from files opened in all write modes."""
     if mode[0] == 'r' or mode[0] == 'a':
         with gsd.fl.open(name=tmp_path / 'test_read_write.gsd',
-                         mode='wb',
+                         mode='w',
                          application='test_read_write',
                          schema='none',
                          schema_version=[1, 2]):
@@ -961,3 +958,75 @@ def test_read_write(tmp_path, mode):
             data10 = f.read_chunk(frame=i, name='data10')
             assert data1[0] == i
             assert data10[0] == i * 10
+
+
+@pytest.mark.parametrize('n_flush', [0, 1, 2])
+def test_flush(tmp_path, open_mode, n_flush):
+    """Test flush."""
+    data = numpy.array([1, 2, 3, 4, 5, 10012], dtype=numpy.int64)
+    with gsd.fl.open(name=tmp_path / 'test_flush.gsd',
+                     mode=open_mode.write,
+                     application='test_flush',
+                     schema='none',
+                     schema_version=[1, 2]) as f:
+        f.write_chunk(name='chunk1', data=data)
+        f.end_frame()
+        f.write_chunk(name='chunk2', data=data)
+        f.end_frame()
+        f.write_chunk(name='chunk3', data=data)
+
+        # Ensure that the data is buffered by opening the file with a 2nd
+        # handle read-only and checking it.
+        with gsd.fl.open(name=tmp_path / 'test_flush.gsd',
+                         mode='r') as f_readonly:
+            assert not f_readonly.chunk_exists(frame=0, name='chunk1')
+            assert not f_readonly.chunk_exists(frame=1, name='chunk2')
+            assert f_readonly.nframes == 0
+
+        # 0 calls to flush tests the implicit flush on close, 2 calls to flush
+        # tests that repeated calls are handled properly.
+        for i in range(n_flush):
+            f.flush()
+
+    with gsd.fl.open(name=tmp_path / 'test_flush.gsd',
+                     mode=open_mode.read) as f:
+        assert f.chunk_exists(frame=0, name='chunk1')
+        assert f.chunk_exists(frame=1, name='chunk2')
+
+        # The third chunk is not written because end_frame is not called a
+        # third time.
+
+        assert not f.chunk_exists(frame=2, name='chunk3')
+        assert f.nframes == 2
+
+
+def test_maximum_write_buffer_size(tmp_path, open_mode):
+    """Test maximum_write_buffer_size."""
+    with gsd.fl.open(name=tmp_path / 'test_maximum_write_buffer_size.gsd',
+                     mode=open_mode.write,
+                     application='test_maximum_write_buffer_size',
+                     schema='none',
+                     schema_version=[1, 2]) as f:
+
+        assert f.maximum_write_buffer_size > 0
+        f.maximum_write_buffer_size = 1024
+        assert f.maximum_write_buffer_size == 1024
+
+        with pytest.raises(RuntimeError):
+            f.maximum_write_buffer_size = 0
+
+
+def test_index_entries_to_buffer(tmp_path, open_mode):
+    """Test index_entries_to_buffer."""
+    with gsd.fl.open(name=tmp_path / 'test_index_entries_to_buffer.gsd',
+                     mode=open_mode.write,
+                     application='test_index_entries_to_buffer',
+                     schema='none',
+                     schema_version=[1, 2]) as f:
+
+        assert f.index_entries_to_buffer > 0
+        f.index_entries_to_buffer = 1024
+        assert f.index_entries_to_buffer == 1024
+
+        with pytest.raises(RuntimeError):
+            f.index_entries_to_buffer = 0
