@@ -15,11 +15,17 @@
 
 int main(int argc, char** argv) // NOLINT
     {
-    const size_t n_keys = 65534;
-    const size_t n_frames = 1000;
-    const size_t key_size = 1;
+    const size_t n_keys = 16;
+    const size_t n_frames = 100;
+    const size_t key_size = static_cast<const size_t>(1024) * static_cast<const size_t>(1024);
 
     std::vector<double> data(key_size);
+
+    for (size_t i = 0; i < key_size; i++)
+        {
+        data[i] = (double)i;
+        }
+
     std::vector<std::string> names;
     for (size_t i = 0; i < n_keys; i++)
         {
@@ -62,6 +68,13 @@ int main(int argc, char** argv) // NOLINT
 
     const double us = 1e-6;
     std::cout << "Write time: " << time_per_key / us << " microseconds/key." << std::endl;
+    std::cout << "Write time: " << time_per_key / us * n_keys << " microseconds/frame."
+              << std::endl;
+
+    const double mb_per_second
+        = double(key_size * 8 + static_cast<const size_t>(32) * static_cast<const size_t>(2))
+          / 1048576.0 / time_per_key;
+    std::cout << "MB/s: " << mb_per_second << " MB/s." << std::endl;
 
     gsd_close(&handle);
 
